/*****************************************************************************
 *
 * LCD Driver for PIC24.
 *
 *****************************************************************************
 * FileName:        lcd.c
 * Dependencies:    p24FJ128GA010.h
 * Processor:		PIC24FJ128GA010
 * IDE :       	    MPLAB v8.10 or higher
 * Compiler :       MPLAB C30 v3.10 or higher
 * HW platform :    EXPLORER16 (DM240001)
 * Plug-In Module : PIC24FJ128GA010 PIM Plug-In Module (MA240011)         
 * Company:         Microchip Technology Incorporated
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the "Company") is intended and supplied to you, the Company's
 * customer, for use solely and exclusively with products manufactured
 * by the Company. 
 *
 * The software is owned by the Company and/or its supplier, and is 
 * protected under applicable copyright laws. All rights are reserved. 
 * Any use in violation of the foregoing restrictions may subject the 
 * user to criminal sanctions under applicable laws, as well as to 
 * civil liability for the breach of the terms and conditions of this 
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES, 
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED 
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT, 
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR 
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 *
 * A simple LCD driver for LCDs interface through the PMP
 * 
 *
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Ross Fosler          08/13/04    ...	
 * Brant Ivey           7/3/07      Updated for v2.0 of class
 * Alain Sorin          7/Dec/07    Updated for v3.10 of 204_ADV class
 * Stu Chandler         12/20/08    Updated for both 3V & 5V LCds
 * Alain Sorin          28/Feb/09   Updated for v4.00 of MCU_3122 class (renamed)
 *****************************************************************************/

#include	<p24FJ128GA010.h>


// Delay loops for LCD routines.  

// Define Fosc for HS Oscillator
// Delay loop is 18uS = 72 Tcy = 144 Tosc
#define Fosc 8000000                      // HS Fosc = 8MHz

// Define a fast instruction execution time in terms of loop time
#define LCD_F_INSTR     (unsigned int)(Fosc/2600000)   // 50uS

// Define a slow instruction execution time in terms of loop time
#define LCD_S_INSTR     (unsigned int)(Fosc/72000)     //  2mS

// Define the startup time for the LCD in terms of loop time 
#define LCD_STARTUP     (unsigned int)(Fosc/2880)      // 50mS


#define		_LCD_IDLE(__cnt)	_uLCDstate = 1; _uLCDloops = __cnt;
#define		_LCD_INIT(__cnt)	_uLCDstate++; _uLCDloops = __cnt;

unsigned int	_uLCDloops;
unsigned int	_uLCDstate;
unsigned int 	_uLCDchar;


// Process LCD events
void LCDProcessEvents(void)
{
	switch (_uLCDstate) {
		case 1:								// *** wait *** 
			if (_uLCDloops) _uLCDloops--;
			else _uLCDstate = 0;
			break;
		case 2:								// *** init ***

			//********************************************************************************
			//  STEP 1:
			//	Configure PMCON (8 bits data): PMP enabled, address/data not multiplexed,  
			//	PMBE disabled, PMENB enabled, RD/~WR enabled, no CS,   
			//  PMENB active high, PMRD/~PMWR active high.
			//********************************************************************************/
		    // ### your code here ###	     // ###

			//********************************************************************************
			//	STEP 2:
			//	Configure PMMODE: No interrupt (polling Busy flag), inc/dec off, 8 bit mode, 
			//  combined read/write with byte enable signals, assume maximum number of wait 
			//	states for beginning, middle and final phase of PMENB strobe  
			//  N.B. : in actual systems, the number of wait states need to be optimised 
			//  depending upon the LCD electrical specification and the MCU operating frequency.
			//********************************************************************************/
		    // ### your code here ###	     // ###

			//********************************************************************************
			//	STEP 3:	
			//	Configure PMAEN: Enable only A0 to control RS and disable all  
			//	other PMP address pins and CS.
			//********************************************************************************/
		    // ### your code here ###	     // ###

			//********************************************************************************
			// 	STEP 4:
			//	Configure PMADDR: A0 selects type of instruction, either command or data.
			//	This is a command so A0 should be low.
			//********************************************************************************/
		    // ### your code here ###	     // ###
		
		//********************************************************************************

			_uLCDstate = 64;				// Set the next state
			_uLCDloops = LCD_STARTUP;		// Set the entry delay
			break;
		case 3:								// *** put ***
			_LCD_IDLE(LCD_F_INSTR);
			PMADDR = 0x0001;
			PMDIN1 = _uLCDchar;
			break;
		case 4:								// *** clear ***
			_LCD_IDLE(LCD_S_INSTR);
			PMADDR = 0x0000;
			PMDIN1 = 0b00000001;
			break;
		case 5:								// *** home ***
			_LCD_IDLE(LCD_S_INSTR);
			PMADDR = 0x0000;
			PMDIN1 = 0b00000010;
			break;	
		case 6:								// *** command ***
			_LCD_IDLE(LCD_F_INSTR);
			PMADDR = 0x0000;
			PMDIN1 = _uLCDchar;
			break;

		// This is the LCD init state machine
		case 64:							// Standard delay states for the init
		case 66:
		case 68:
		case 70:							// Programmable delay loop
		case 72:
			if (_uLCDloops) _uLCDloops--;
			else _uLCDstate++;
			break;
		case 65:
			_LCD_INIT(LCD_F_INSTR);
			PMDIN1 = 0b00111000;			// Set the default function
			break;
		case 67:
			_LCD_INIT(LCD_F_INSTR);
			PMDIN1 = 0b00001100;			// Set the display control
			break;
		case 69:	
			_LCD_INIT(LCD_S_INSTR);
			PMDIN1 = 0b00000001;			// Clear the display
			break;
		case 71:
			_LCD_INIT(LCD_S_INSTR);
			PMDIN1 = 0b00000110;			// Set the entry mode
			break;
		default:
			_uLCDstate = 0;
			break;
	}
}
